/* Computing of the values of the lines */
allLineValues( Solutions):-
	collectAllCrossings(ListOfAllCrossings),
	bagof( ListOfAllCrossings, lineValues(ListOfAllCrossings), Solutions).

lineValues([ ] ).
lineValues( [ cross( _Name, Type, Lines) | RestOfCrossings] ) :-
	propagateForOneCrossing( cross( _Name, Type, Lines) ),
	propagateForListOfLines( Lines, RestOfCrossings ),
	lineValues(RestOfCrossings).

% Values for one crossing	
propagateForOneCrossing(cross(_Name,Type, Lines)) :-
	typeRotationInvariant(Type, Lines).

% For one crossing: The values of the lines are propagated to the
% lines of all other crossings

propagateForListOfLines([ ], _Crossings).
propagateForListOfLines([ First| Rest], Crossings) :-
	propagateForOneLine( First, Crossings),	
	propagateForListOfLines( Rest, Crossings).

propagateForOneLine( _, [ ] ).
propagateForOneLine( (Line, Value), [ cross( _ ,  _ , List) | Rest ]):-
	memberPair( ( Line, Value), List), !, 
	propagateForOneLine( (Line, Value), Rest).

propagateForOneLine( LineValue, [ _  | Rest ] ) :-
	propagateForOneLine( LineValue, Rest). 


/* Additional predicats */
/* This predicate collect all crossing in the images */

collectAllCrossings(ListOfAllCrossings) :-	
	bagof( cross(Name,Type,Lines),cross(Name, Type, Lines ), ListOfAllCrossings).

typeRotationInvariant( Type, Lines) :- type( Type, Lines).

memberPair( _, [  ]). 
memberPair( X, [ X| L]) :-  !,  
	memberPair( X, L).
memberPair(X , [ _Item |L] ) :- 
	memberPair( X, L).

/* Crossings-knowledgebase for Example on page 167*/
% L-Type
type(l, [( _L1, ' - ')  ,  ( _L2,  ' <- ')] ).
type(l, [( _L1, ' <- ') , ( _L2,  ' - ')] ).
type(l, [( _L1, ' -> ') , ( _L2,  ' - ')] ).
type(l, [( _L1, ' - ')  ,  ( _L2,  ' -> ')] ).
type(l, [( _L1, ' -> ') , ( _L2,  ' <- ')] ).
type(l, [( _L1, ' <- ') , ( _L2,  ' -> ')] ).

% Y-Type 
type(y, [( _L1, ' + ') , ( _L2,  ' <- '), ( _L3, ' <- ')] ).
type(y, [( _L1, ' + ') , ( _L2,  ' - '), ( _L3, ' - ')] ).
type(y, [( _L1, ' - ') , ( _L2,  ' + '), ( _L3, ' - ')] ).
type(y, [( _L1, ' -> ') , ( _L2,  ' + '), ( _L3, ' -> ')] ).

% arrow-Type 
type(arrow, [( _L1, ' + ') , ( _L2,  ' <- '), ( _L3, ' <- ')] ).
type(arrow, [( _L1, ' + ') , ( _L2,  ' - '), ( _L3, ' - ')] ).
type(arrow, [( _L1, ' - ') , ( _L2,  ' + '), ( _L3, ' - ')] ).
type(arrow, [( _L1, ' - ') , ( _L2,  ' - '), ( _L3, ' + ')] ).
type(arrow, [( _L1, ' -> ') , ( _L2,  ' + '), ( _L3, ' -> ')] ).
type(arrow, [( _L1, ' <- ') , ( _L2,  ' <- '), ( _L3, ' + ')] ).


